{*********************************************************}
{*          Web server application components            *}
{*                     version 2.0                       *}
{*         Copyright (c) 2000-2002  S.Perevoznyk         *}
{*                 All rights reserved.                  *}
{*********************************************************}
{$IFDEF VER140}
{$WARN SYMBOL_DEPRECATED OFF }
{$ENDIF}

unit psvWebControls;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Jpeg, ComCtrls, Chart;

type
  TpsvWebControl = class(TComponent)
  private
    FWindowHandle : THandle;
    FWidth: integer;
    FHeight: integer;
    FControl: TWinControl;
    procedure SetOutput(Value: TBitmap); virtual;
    procedure SetHeight(Value: integer);
    procedure SetWidth(Value: integer);
    procedure WindProc(var Msg: TMessage);
  protected
    function  GetContentAsStream: TStream; virtual;
    function  GetOutput: TBitmap; virtual;
    property  Control : TWinControl read FControl write FControl;
    property  WindowHandle : THandle read FWindowHandle;
  public
    constructor Create(AOwner : TComponent); override;
    property ContentAsStream: TStream read GetContentAsStream;
    destructor Destroy; override;
  published
    property Width: integer read FWidth write SetWidth default 200;
    property Height: integer read FHeight write SetHeight default 150;
    property Output: TBitmap read GetOutput write SetOutput stored false;
  end;

  TpsvWebChart = class(TpsvWebControl)
  private
    FChart : TChart;
    procedure SetChart(const Value : TChart);
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Chart : TChart read FChart write SetChart;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('FastWeb',[TpsvWebChart]);
end;

destructor TpsvWebControl.Destroy;
begin
  if Assigned(FControl) then
   FControl.Free;
  DeallocateHWnd(FWindowHandle);
  Inherited;
end;

procedure TpsvWebControl.SetOutput(Value: TBitmap);
begin
  // do nothing
end;

procedure TpsvWebControl.SetHeight(Value: integer);
begin
  if Value <> FHeight then
  begin
    FHeight := Value;
    if Assigned(FControl) then
      FControl.Height := FHeight;
  end;
end;

procedure TpsvWebControl.SetWidth(Value: integer);
begin
  if Value <> FWidth then
  begin
    FWidth := Value;
    if Assigned(FControl) then
      FControl.Width := FWidth;
  end;
end;



function TpsvWebControl.GetOutput: TBitmap;
var
  Dummy: TForm;
begin
  Result := TBitmap.Create;
  if Not Assigned(FControl) then
   Exit;
  try
    Dummy := TForm.Create(nil);
    try
      FControl.Parent := Dummy;
      Result.Width := FControl.Width;
      Result.Height := FControl.Height;
      Result.Canvas.Lock;
      try
        (FControl as TWinControl).PaintTo(Result.Canvas.Handle, 0, 0);
      finally
        Result.Canvas.Unlock;
      end;
    finally
      FControl.Parent := nil;
      Dummy.Free;
    end;
  except
    Result.Free;
    raise;
  end;
end;

function TpsvWebControl.GetContentAsStream: TStream;
var
  Jpg: TJpegImage;
begin
  Result := TMemoryStream.Create;
    try
      Jpg := TJpegImage.Create;
      try
        Jpg.Assign(Output);
        Jpg.SaveToStream(Result);
        Result.Position := 0;
      finally
        Jpg.Free;
      end;
  except
    Result.Free;
    raise;
  end;
end;

procedure TpsvWebControl.WindProc(var Msg: TMessage);
begin
 with msg do
  Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;


constructor TpsvWebControl.Create(AOwner: TComponent);
begin
  inherited;
  FWindowHandle := AllocateHWnd(WindProc);
  FWidth := 200;
  FHeight := 150;
end;

{ TpsvWebChart }

constructor TpsvWebChart.Create(AOwner: TComponent);
begin
  inherited;
  FChart := TChart.Create(Self);
  FChart.ParentWindow := FWindowHandle;
  FControl := FChart;
  FControl.Width := fWidth;
  fControl.Height := FHeight;
end;

destructor TpsvWebChart.Destroy;
begin
  inherited;
end;

procedure TpsvWebChart.SetChart(const Value: TChart);
begin
  FChart.Assign(Value);
end;

end.
